$PG
$PW=130
$title(MultiMax ALTERNATIVE MIDI CONTROLLER - Version 0.3)
************************************************************
************************************************************
***                    MULTIMAX.ASM                      ***
**                     Version  0.3                       **
***      Monophonic Output and Polyphonic Output         ***
************************************************************
************************************************************

; Version 0.2
; -----------
; 1. Changed tick interval to 2 ms (500 Hz kybd sampling interval)
;    in order to change keyboard velocity response.  Switch scans
;    remain at 10 ms intervals (5 ticks @ 2 ms).
; 2. Octave priority changed from <+2 +1 -1 -2> to  <+2 -2 +1 -1>
;    for the monophonic kybd only.
; 3. Sensitivity window table settings changes from 15ms - 255ms
;    evenly distributed to 10ms - 45ms then 55ms - 125ms.  Default
;    is changed to 30ms from 80ms.
; 4. Note-on now appears before note-off.
; 5. Additional button called TRANSPOSE MODE which, when enabled,
;    turns the trans buttons into more controllers (50h...53h).
;    (Mono modes only)
;
; Version 0.3
; -----------
; 1. Reversed Version 0.2 - #1.  Tick interval 1 ms again.
; 2. Repaired NOTEON routine which failed to update lastvel.
; 3. Re-altered sensitivity window table settings for 10ms - 45ms
;    then 55 - 125 ms EXACTLY.

	      DEFSEG MIDICODE, ABSOLUTE
	      SEG    MIDICODE

************************************************************
*                      EXTERNAL I/O                        *
************************************************************

KEYS          EQU    8000H        ;KEYBOARD LATCH (74HC573)
ACIASR        EQU    0FFF9H       ;DRYICE UART STATUS REGISTER
ACIADR        EQU    0FFFBH       ;DRYICE UART DATA REGISTER

************************************************************
*                      INTERNAL RAM                        *
************************************************************

;------------------- REGISTER BANKS: 00 to 1F --------------

;00 to 07: REGISTER BANK 0 - MAIN PROGRAM
;                            R0, (no other registers used)
;08 to 0F: REGISTER BANK 1 - SERIAL INTERRUPT SERVICE ROUTINE
;                            R0, (no other registers used)
;10 to 17: unused
;18 to 1F: unused

;------------------- BIT ADDRESSABLE: 20 to 2F -------------

THISKVAL      EQU    20H          ;KEY COMBINATION BEING
                                  ;ESABLISHED THIS PASS

CTRL0         EQU    21H          ;CTRL0.0 = CONTROLLER 40H
                                  ;CTRL0.1 = CONTROLLER 41H
                                  ;   :
                                  ;CTRL0.7 = CONTROLLER 47H

CTRL1         EQU    22H          ;CTRL1.0 = CONTROLLER 48H
                                  ;CTRL1.1 = CONTROLLER 49H
                                  ;   :
                                  ;CTRL1.7 = CONTROLLER 4FH

ctrl2         equ    23h          ;ctrl2.0 = controller 50h
                                  ;ctrl2.1 = controller 51h
                                  ;ctrl2.2 = controller 52h
                                  ;ctrl2.3 = controller 53h

MODE          EQU    24H          ;MODE.0 = 0 UNI-RETRIGGER
                                  ;MODE.0 = 1 UNI-LEGATO
                                  ;mode.1 = 0 normal transpose
                                  ;mode.1 = 1 transpose buttons
                                  ;           are more controllers
                                  ;bits 2 - 7 not used

;25H to 2FH: unused

;------------------- BYTE ADDRESSABLE: 30 to 5D ------------

VEL0          EQU    30H          ;VELOCITY COUNTERS:
VEL1          EQU    31H          ;  KEY UP VELOCITY  =  7F
VEL2          EQU    32H          ;  GOING DOWN =  7F to 01
VEL3          EQU    33H          ;  KEY DOWN VELOCITY = 00
VEL4          EQU    34H          ;

KEY0          EQU    35H          ;KEY VELOCITY REGISTERS:
KEY1          EQU    36H          ;  CONTAINS LAST VELOCITY
KEY2          EQU    37H          ;  FROM MOST RECENT KEY
KEY3          EQU    38H          ;  DOWN
KEY4          EQU    39H

LASTVEL       EQU    3AH          ;LAST VELOCITY USED
LASTOCT       EQU    3BH          ;LAST OCTAVE VALUE USED

CURRKVAL      EQU    3CH          ;CURRENT KEY COMBINATION
                                  ;BEING CHECKED FOR DEBOUNCING

LASTNOTE      EQU    3DH          ;LAST NOTE SENT (BINARY KEY VAL)

TICK          EQU    3EH          ;10 mS TICK (SWITCH COUNTER)

WINDOW        EQU    3FH          ;KEYBOARD SENSITIVITY WINDOW
                                  ;  ACTUAL DEBOUNCE COUNTER FOR
                                  ;  VALID KEYBOARD COMBINATION
USERSENS      EQU    40H          ;USER DEFINED SENSITIVITY WINDOW
                                  ;  FOR MULTI-KEY DEBOUNCE

WARMBOOT      EQU    41H          ;AFTER BOOT-UP, #5A PLACED HERE
                                  ;TO STOP UNI-RETRIGGER FROM
                                  ;BEING THE DEFAULT FROM WARM RESET

;41H to 4DH: unused

;------------------- FIFO AREA: 4E to 67 -------------------

QHEAD         EQU    4EH          ;POINTER TO ENTRY INTO QUEUE
QTAIL         EQU    4FH          ;POINTER TO EXIT FROM QUEUE
QUEBOT        EQU    50H          ;BOTTOM OF CIRCULAR BUFFER
QUETOP        EQU    67H          ;TOP (END) OF CIRCULAR BUFFER

;------------------- STACK AREA: 68 to 7F ------------------

STACK         EQU    67H          ;MUST EQUAL HIGHEST
                                  ;INTERNAL RAM LOCATION
                                  ;STACK GROWS UP FROM
                                  ;NEXT HIGHER LOCATION

************************************************************
*               DEFINED INTERNAL REGISTERS                 *
************************************************************

;------------------- BITWIDE -------------------------------

;------------------- BYTEWIDE ------------------------------

************************************************************
*                                                          *
*                         VECTORS                          *
*                                                          *
************************************************************

eprom         equ    1            ;for conditional assembly

              if     eprom eq 1
USERRAM       EQU    0000H        ;For EPROM version
              else
USERRAM       EQU    4000H        ;For DryIce version
              endif

************************************************************
*                         RESET                            *
************************************************************

              ORG    USERRAM
RESET1        JMP    INIT

************************************************************
*          TIMER 0 OVERFLOW INTERRUPT (TICK)               *
************************************************************

              ORG    USERRAM+0BH
              SJMP   TF0_ISR      ;ACTUAL CODE FOUND BELOW

************************************************************
*          SERIAL I/F INTERRUPT SERVICE ROUTINE            *
************************************************************

              ORG    USERRAM+23H
TX_ISR        PUSH   ACC          ;SAVE REGISTERS
              PUSH   PSW          ;MAIN PROGRAM USES REG BANK 0
              MOV    PSW,#08H     ;USE REGISTER BANK 1
DEQUEUE       MOV    A,QTAIL
              CJNE   A,QHEAD,DQ_1 ;TEST IF QUEUE EMPTY (H=T)
              CLR    ES           ;Q EMPTY, DISABLE SERIAL INT'S
                                  ;(BUT DO NOT CLEAR: WILL BE
                                  ; USED TO RE-START IN MAIN)
              SJMP   TX_RET       ;AND EXIT
DQ_1          CLR    TI           ;INTERRUPT ACKNOWLEDGE
                                  ;(ie. CLEAR REQUEST)
              INC    A            ;UPDATE TAIL POINTER
              CJNE   A,#QUETOP+1,DQ_2  ;OVER THE TOP?
              MOV    A,#QUEBOT         ;RESET TO START
DQ_2          MOV    R0,A         ;R0 IS POINTER
              MOV    SBUF,@R0     ;SEND THE NEXT BYTE
              MOV    QTAIL,A      ;AND SAVE THE LAST TAIL
TX_RET        POP    PSW          ;RESTORE BANK 0 & FLAGS
              POP    ACC
              RETI                ;AND RETURN

************************************************************
*          TIMER 0 OVERFLOW INTERRUPT (TICK)               *
************************************************************

TF0_ISR       PUSH   ACC
              MOV    TL0,#LOW (-1000) ;TMR 0 RELOAD VALUE = -1000
              MOV    TH0,#HIGH(-1000) ;  FOR 1 ms 'TICK'
              CPL    T0           ;SHOW ACTIVITY ON PIN 14
              MOV    A,VEL0       ;GET VELOCITY OF KEY 0
              JZ     VEL0DONE     ;IF = 0 CANNOT DECREMENT
              DJNZ   VEL0,VEL0DONE ;VELOCITY - 1
              INC    VEL0          ;MIN VELOCITY == 01
VEL0DONE      MOV    A,VEL1
              JZ     VEL1DONE
              DJNZ   VEL1,VEL1DONE
              INC    VEL1
VEL1DONE      MOV    A,VEL2
              JZ     VEL2DONE
              DJNZ   VEL2,VEL2DONE
              INC    VEL2
VEL2DONE      MOV    A,VEL3
              JZ     VEL3DONE
              DJNZ   VEL3,VEL3DONE
              INC    VEL3
VEL3DONE      MOV    A,VEL4
              JZ     VEL4DONE
              DJNZ   VEL4,VEL4DONE
              INC    VEL4
VEL4DONE      POP    ACC
              SETB   F0           ;SET 1 mS FLAG TRUE
              RETI

************************************************************
*                                                          *
*                         PROGRAM                          *
*                                                          *
************************************************************

MTG           DB     " *** Copyright (C) 1992, "
              DB     "Music Technologies Group *** "
              DB     "Edmonton, AB, Canada "
              DB     "Written by Grant P. Beattie, P. Eng."
              DB     " All rights reserved. "

************************************************************
*                      INITIALIZATION                      *
************************************************************
; Determine the operating mode: If all keys up = MONO
;                               If any key not up = POLY

INIT          MOV    A,WARMBOOT   ;IS THIS THE FIRST POWER-UP?
              CJNE   A,#5AH,COLD  ;  IF <> 5A COLD RESTART
              JMP    WARM         ;  IF == 5A WARM BOOT

;---------- THIS CODE SKIPPED DURING A WARM BOOT -----------
COLD          MOV    WARMBOOT,#5AH ;WAS COLD, BUT NOT ANY MORE
              CLR    MODE.0        ;SET DEFAULT MODE = RETRIGGER
; version 0.2
              clr    mode.1        ;default = transpose
              clr    a
              mov    ctrl2,a       ;all 4 controllers off
;------------------- END OF SKIPPED CODE -------------------


WARM          MOV    P1,#0FEH     ;BRING 'BREAK' LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              CJNE   A,#0E0H,DOPOLY
              JMP    MONOINIT       ;IF A=E0, ALL UP, DO MONO
DOPOLY        MOVX   A,@DPTR        ;READ IT AGAIN
              CJNE   A,#0E0H,DOPOLY ;WAIT FOR ALL KEYS UP
              JMP    POLYINIT       ;WHEN ALL UP GO TO POLY


MONOINIT      MOV    PSW,#00H     ;USE LOWEST SET OF REGISTERS
              MOV    SP,#STACK    ;SET SP ABOVE INT VARS

;-------------temporary stuff for testing only--------------
;              MOV    A,#0FFH      ;INITIALIZE ALL INTERNAL
;              MOV    R0,#07FH     ;  RAM TO FF FOR TESTING
;RAM_FILL      MOV    @R0,A        ;  PURPOSES
;              DJNZ   R0,RAM_FILL
;-----------------------------------------------------------

FASTSENS      MOV    P1,#0F7H     ;CHECK USER DEFINED SENSITIVITY
              MOV    DPTR,#KEYS
              MOVX   A,@DPTR
              CPL    A            ;NOW ACTIVE HIGH
              JZ     SLOWSENS     ;NOTHING IN THIS ROW
; version 0.3
              MOV    USERSENS,#5  ;5+5 = 10ms
SETFAST       PUSH   ACC
              MOV    A,USERSENS
              ADD    A,#5         ;10,15,20,25,30,35,40,45
;
              MOV    USERSENS,A
              POP    ACC
              RRC    A            ;LSB TO CF
              JNC    SETFAST      ;UNTIL A ONE FOUND
FASTUP        MOVX   A,@DPTR
              CJNE   A,#0FFH,FASTUP ;WAIT FOR ALL BUTTONS UP
              JMP    SAVESENS

SLOWSENS      MOV    P1,#0EFH
              MOVX   A,@DPTR
              CPL    A
              JZ     NO_SENS      ;NO BUTTONS AT ALL, USE DEFAULT
; version 0.3
              MOV    USERSENS,#45 ;45 + 10 = 50 ms
SETSLOW       PUSH   ACC
              MOV    A,USERSENS
              ADD    A,#10        ;55,65,75,85,95,105,115,125
;
              MOV    USERSENS,A
              POP    ACC
              RRC    A
              JNC    SETSLOW
SLOWUP        MOVX   A,@DPTR
              CJNE   A,#0FFH,SLOWUP
              JMP    SAVESENS
; version 0.3
NO_SENS       MOV    A,#30        ;30ms POWER-UP DEFAULT
;
              MOV    USERSENS,A
SAVESENS      NOP

              CLR    A
              MOV    KEY0,A       ;INITIALIZE ALL KEYS AS
              MOV    KEY1,A       ;  BEING UP
              MOV    KEY2,A
              MOV    KEY3,A
              MOV    KEY4,A

              MOV    WINDOW,A     ;RESET DEBOUNCE COUNTER
              MOV    LASTNOTE,A   ;NO LAST MIDI NOTE
              MOV    LASTVEL,A    ;NO LAST MIDI VELOCITY
              MOV    THISKVAL,A   ;NO KEY COMBINATION
              MOV    CURRKVAL,A   ;    "
              MOV    LASTOCT,A    ;NO ACTIVE OCTAVE
              MOV    TICK,A
              MOV    CTRL0,A      ;NO CONTROLLERS
              MOV    CTRL1,A
              MOV    A,#7FH       ;SET ALL INITIAL NOTE
              MOV    VEL0 ,A      ;  VELOCITY BYTES TO MAX
              MOV    VEL1 ,A      ;  VELOCITY = 7FH
              MOV    VEL2 ,A
              MOV    VEL3 ,A
              MOV    VEL4 ,A

              MOV    TMOD,#21h    ;DISABLE GATE, TIMER MODE
                                  ;TIMER 1: MODE 2 (8 BIT)
                                  ;TIMER 0: MODE 1 (16 BIT)
              MOV    TL1,#0FFh    ;TMR 1 RELOAD VALUE=255
              MOV    TH1,#0FFh    ;  SINCE SMOD=0 (MIDI BAUD)
              MOV    TL0,#LOW (-1000) ;TMR 0 RELOAD VALUE = -1000
              MOV    TH0,#HIGH(-1000) ;  FOR 1 kHz 'TICK'
              MOV    TCON,#50h    ;BOTH TIMERS RUNNING
              MOV    SCON,#70h    ;MODE 1, 8 BIT MIDI UART

              MOV    QHEAD,#QUETOP ;INITIALIZE QUEUE POINTERS
              MOV    QTAIL,#QUETOP ;  FOR EMPTY QUEUE
              SETB   EA           ;ENABLE INTERRUPTS
              SETB   ES           ;ENABLE SERIAL INTERRUPTS
              SETB   TI           ;CALL TX ROUTINE TO ARM QUEUE
                                  ;(DOESN'T TRANSMIT ANYTHING
                                  ; BECAUSE QUEUE IS EMPTY)
              SETB   ET0          ;ENABLE TIMER 0 INTERRUPTS
                                  ;(TICK TIMER)

************************************************************
*                     KEYBOARD SCAN                        *
************************************************************
;.---------------------------------------------------------.
;|   Check to see which keys are in the UP position.  If a |
;| key is UP check to see if it was PREVIOUSLY down (VELx=0|
;| for a key down). If it was DOWN then it's status must   |
;| now show that it is up.  If the key is up and was NOT   |
;| previously down, reset velocity (VELx = 7F).            |
;`---------------------------------------------------------'

BREAKTEST     CLR    F0           ;USE PSW.5 AS 1 mS FLAG
                                  ;INDICATOR.

              MOV    P1,#0FEH     ;BRING 'BREAK' LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE LATCH VALUE IN B

BRKTEST0      JB     B.0,BRKTEST1 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL0       ;READ NOTE VALUE
              JNZ    VEL7F_0      ;IF VEL <> 0 RESET TO 7F
              CLR    THISKVAL.0   ;KEY 0 IS OFF
VEL7F_0       MOV    VEL0,#7FH    ;RESET NOTE 0 == 7F

BRKTEST1      JB     B.1,BRKTEST2 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL1       ;READ NOTE VALUE
              JNZ    VEL7F_1      ;IF VEL <> 0 RESET TO 7F
              CLR    THISKVAL.1   ;KEY 1 IS OFF
VEL7F_1       MOV    VEL1 ,#7FH   ;RESET NOTE 1 == 7F

BRKTEST2      JB     B.2,BRKTEST3 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL2       ;READ NOTE VALUE
              JNZ    VEL7F_2      ;IF VEL <> 0 RESET TO 7F
              CLR    THISKVAL.2   ;KEY 2 IS OFF
VEL7F_2       MOV    VEL2 ,#7FH   ;RESET NOTE 2 == 7F

BRKTEST3      JB     B.3,BRKTEST4 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL3       ;READ NOTE VALUE
              JNZ    VEL7F_3      ;IF VEL <> 0 RESET TO 7F
              CLR    THISKVAL.3   ;KEY 3 IS OFF
VEL7F_3       MOV    VEL3 ,#7FH   ;RESET NOTE 3 == 7F

BRKTEST4      JB     B.4,MAKETEST ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL4       ;READ NOTE VALUE
              JNZ    VEL7F_4      ;IF VEL <> 0 RESET TO 7F
              CLR    THISKVAL.4   ;KEY 4 IS OFF
VEL7F_4       MOV    VEL4 ,#7FH   ;RESET NOTE 4 == 7F

;.---------------------------------------------------------.
;|   Check to see which keys are in the DOWN position.  If |
;| a key is NOT down, do nothing; the velocity will be man-|
;| aged by the BREAK routine (above) and the 1ms interrupt.|
;| If the key is DOWN and VELx=0, the key status is already|
;| ON. If the key is DOWN but VELx<>0 then turn key status |
;| to ON, set KEYx==VELx and finally force the VELx==0.    |
;`---------------------------------------------------------'

MAKETEST      MOV    P1,#0FDH     ;BRING 'MAKE' LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE LATCH VALUE IN B

MKTEST0       JB     B.0,MKTEST1  ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL0       ;READ NOTE VALUE
              JZ     MKTEST1      ;IF VEL = 0 ALREADY ON
NOTEON0       MOV    KEY0,A       ;KEY 0 IS ON
              SETB   THISKVAL.0
              CLR    A
              MOV    VEL0,A       ;RESET VELOCITY = 00

MKTEST1       JB     B.1,MKTEST2  ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL1       ;READ NOTE VALUE
              JZ     MKTEST2      ;IF VEL = 0 ALREADY ON
NOTEON1       MOV    KEY1,A       ;KEY 1 IS ON
              SETB   THISKVAL.1
              CLR    A
              MOV    VEL1,A       ;RESET VELOCITY = 00

MKTEST2       JB     B.2,MKTEST3  ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL2       ;READ NOTE VALUE
              JZ     MKTEST3      ;IF VEL = 0 ALREADY ON
NOTEON2       MOV    KEY2,A       ;KEY 2 IS ON
              SETB   THISKVAL.2
              CLR    A
              MOV    VEL2,A       ;RESET VELOCITY = 00

MKTEST3       JB     B.3,MKTEST4  ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL3       ;READ NOTE VALUE
              JZ     MKTEST4      ;IF VEL = 0 ALREADY ON
NOTEON3       MOV    KEY3,A       ;KEY 3 IS ON
              SETB   THISKVAL.3
              CLR    A
              MOV    VEL3,A       ;RESET VELOCITY = 00

MKTEST4       JB     B.4,KEYCALC  ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL4       ;READ NOTE VALUE
              JZ     KEYCALC      ;IF VEL = 0 ALREADY ON
NOTEON4       MOV    KEY4,A       ;KEY 4 IS ON
              SETB   THISKVAL.4
              CLR    A
              MOV    VEL4,A       ;RESET VELOCITY = 00

;.---------------------------------------------------------.
;|   If the key combination from this pass is the same as  |
;| the current note playing, then no note-on need be sent. |
;`---------------------------------------------------------'

KEYCALC       MOV    A,THISKVAL   ;CHECK KYBD COMBINATION VALUE

              CJNE   A,LASTNOTE,CHKCURR  ;IF KEY VAL IS SAME AS
              JMP    CHECKSW             ;NOTE THAT IS SOUNDING
                                         ;THEN NOTHING TO DO

;.---------------------------------------------------------.
;|   If the key combination from this pass is the same as  |
;| the the key combination from last pass, update the sens-|
;| itivity counter until fully debounced.  Once debounce   |
;| value has been reached issue a new note on.             |
;`---------------------------------------------------------'

CHKCURR       CJNE   A,CURRKVAL,RESETCUR ;NEW COMBINATION?
                                    ;YES, RESET CURRKVAL AND
                                    ;     SENSITIVITY COUNT
              INC    WINDOW         ;NO, SAME AS LAST PASS
              MOV    A,WINDOW       ;THEREFORE INC COUNTER
              CJNE   A,USERSENS,CHECKSW ;COUNT = SENSITIVITY?
                                    ;  NO, DO NOTHING
                                    ;  YES, SOUND OFF


;.---------------------------------------------------------.
;|   Determine a velocity. Check each key for an instance  |
;| of a new key going down. Take the velocity of the first |
;| new key down discovered.  If no new key has gone down   |
;| it must be only keys up, therefore use last issued velo-|
;| city. If no keys are down at all, it must be a note-off.|
;`---------------------------------------------------------'

PICKVEL       MOV    CURRKVAL,THISKVAL ;CURRENT KEY WILL NOW
                                       ;BE KEY FROM THIS PASS
              MOV    A,CURRKVAL   ;USE ACC FOR ZERO TEST
              JNZ    CHK_DOWN     ;SOME KEYS STILL DOWN
                                  ;  DETERMINE IF ANY NEW
                                  ;  KEYS ARE DOWN
              CALL   NOTEON       ;IF ALL KEYS ARE NOW UP
                                  ;  ISSUE ONLY A NOTE-OFF
                                  ;  (SINCE ACC A = 0)
              JMP    CHECKSW


RESETCUR      MOV    CURRKVAL,A   ;SAVE NEW CURRENT KYDB VALUE
              CLR    A            ;  AND RESET SENSITIVITY
              MOV    WINDOW,A     ;  COUNTER TO ZERO
              JMP    CHECKSW


CHK_DOWN      JNB    MODE.0,UNIRETRIGGER ;RETRIGGER MODE
UNILEGATO     MOV    A,LASTNOTE   ;IF NO PREVIOUS NOTE THEN
              JZ     UNIRETRIGGER ;NEED A VELOCITY (SAME
                                  ;   AS RETRIGGER CASE)
              JMP    ONLY_UP      ;OTHERWISE UNILEGATO USES
                                  ;LAST VELOCITY UNTIL ALL UP


UNIRETRIGGER  MOV    A,LASTNOTE   ;LOGICAL FUNCTION:
              CPL    A            ; f = (LASTNOTE\)*(CURRKVAL)
              ANL    A,CURRKVAL
              JZ     ONLY_UP      ;IF f=0, NO NEW KEYS ARE
                                  ;  DOWN (ONLY KEYS UP -
                                  ;  USE PREVIOUS VELOCITY)


;     Look through the KEYx table starting at KEY0.  Take first
; velocity that corresponds to that key being newly pressed. R0
; pointer into KEYx velocity table. ACC holds the `f' result
; from the above calculation.  Shift through the `f' byte until
; a 1 is found.

              MOV    R0,#KEY0     ;POINTER TO VELOCITY TABLE
VELSEARCH     RRC    A            ;SHIFT LSB IN CARRY
              JC     THIS_VEL     ;IF = 1, USE CORRESPONDING VEL
              INC    R0           ;POINT TO NEXT VELOCITY
              SJMP   VELSEARCH

THIS_VEL      MOV    B,@R0        ;PLACE VELOCITY IN B
              MOV    A,CURRKVAL   ;PLACE NOTE IN ACC
              CALL   NOTEON
              JMP    CHECKSW

ONLY_UP       MOV    B,LASTVEL    ;VELOCITY IN B
              MOV    A,CURRKVAL   ;NOTE IN A
              CALL   NOTEON
              JMP    CHECKSW

************************************************************
*                   SWITCH SCAN DEBOUNCE                   *
************************************************************
;.---------------------------------------------------------.
;|    Wait for 10 milliseconds to pass between pushbutton  |
;| switch reads. If the 1mS timer has not expired 10 times |
;| then abort the switch reads and instead return to the   |
;| keyboard routine.                                       |
;`---------------------------------------------------------'

CHECKSW       MOV    A,TICK       ;READ SWITCH DEBOUNCE COUNT
              INC    A            ;  TICK += 1
              CJNE   A,#10,NOTYET ;IF TICK <> 10, MUST WAIT
              CLR    A            ;  IF = 10, CHECK SWITCHES
              MOV    TICK,A       ;  AND RESET TICK = 0
              SJMP   PLUS24
NOTYET        MOV    TICK,A       ;DON'T BOTHER WITH SWITCHES
              JMP    WAIT1MS      ; ... BACK TO KEYBOARD


************************************************************
*                 TRANSPOSE SWITCH SCAN                    *
************************************************************
;.---------------------------------------------------------.
;|    Check the TRANSPOSE switches. Once a valid transpose |
;| value is determined, check to see if it corresponds to  |
;| the active transpose value. If is does do nothing, if it|
;| is a new value issue a note-off (if any) and a note-on. |
;`---------------------------------------------------------'


PLUS24        jb     mode.1,control ;version 0.2, if mode is
                                    ;for more controllers, skip
                                    ;transpose code
              MOV    P1,#0FBH     ;BRING TRANSPOSE LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE IT IN B
; version 0.2
              CLR    A            ;SET NEW OCTAVE DEFAULT TO 0
              JB     B.3,minus24  ;SW NOT CLOSED, SKIP TO NEXT
              MOV    A,#24        ;UP 2 OCTAVES
              JMP    COMPARE      ;GO TO COMPARE WITH LASTOCT

minus24       jb     b.0,plus12
              mov    a,#-24       ;down 2 octaves
              jmp    compare

PLUS12        JB     B.2,MINUS12
              MOV    A,#12        ;UP 1 OCTAVE
              JMP    COMPARE

MINUS12       JB     B.1,compare
              MOV    A,#-12       ;DOWN 1 OCTAVE
              JMP    COMPARE
;
COMPARE       CJNE   A,LASTOCT,CHG_NOTE ;IF THERE IS NO CHANGE
              JMP    CONTROL            ;IN THE OCTAVE, SKIP
                                        ;TO CONTROLLER SCAN

CHG_NOTE      MOV    B,A          ;SAVE OCTAVE IN B
              MOV    A,LASTNOTE   ;CHECK TO SEE ANY NOTE
              JNZ    NEW_OCT      ;  YES, MUST CHANGE IT
              MOV    LASTOCT,B    ;UPDATE LASTOCT
              JMP    CONTROL

NEW_OCT       MOV    R0,LASTNOTE  ;SAVE LASTNOTE IN R0
              CLR    A
              CALL   NOTEON       ;KILL OLD NOTE
              MOV    LASTOCT,B    ;UPDATE OCTAVE
              MOV    A,R0         ;LASTNOTE IN ACC
              MOV    B,LASTVEL
              CALL   NOTEON       ;ISSUE NEW NOTE-ON


************************************************************
*                CONTROLLER SWITCH SCAN                    *
************************************************************

CONTROL       MOV    P1,#0F7H     ;BRING Sw0 - Sw7 LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              CPL    A            ;ACTIVE HIGH RESULT
              MOV    B,A          ;SAVE IT IN B

CTRL64        JB     B.0,ON64TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.0,CTRL65 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#40H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.0        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL65
ON64TST       JB     CTRL0.0,CTRL65 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#40H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.0        ;  SHOW CONTROLLER NOW ON

CTRL65        JB     B.1,ON65TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.1,CTRL66 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#41H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.1        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL66
ON65TST       JB     CTRL0.1,CTRL66 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#41H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.1        ;  SHOW CONTROLLER NOW ON

CTRL66        JB     B.2,ON66TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.2,CTRL67 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#42H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.2        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL67
ON66TST       JB     CTRL0.2,CTRL67 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#42H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.2        ;  SHOW CONTROLLER NOW ON

CTRL67        JB     B.3,ON67TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.3,CTRL68 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#43H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.3        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL68
ON67TST       JB     CTRL0.3,CTRL68 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#43H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.3        ;  SHOW CONTROLLER NOW ON

CTRL68        JB     B.4,ON68TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.4,CTRL69 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#44H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.4        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL69
ON68TST       JB     CTRL0.4,CTRL69 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#44H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.4        ;  SHOW CONTROLLER NOW ON

CTRL69        JB     B.5,ON69TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.5,CTRL6A ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#45H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.5        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6A
ON69TST       JB     CTRL0.5,CTRL6A ;NOW ON, DONE IF ON BEFORE
              MOV    A,#45H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.5        ;  SHOW CONTROLLER NOW ON

CTRL6A        JB     B.6,ON6ATST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.6,CTRL6B ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#46H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.6        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6B
ON6ATST       JB     CTRL0.6,CTRL6B ;NOW ON, DONE IF ON BEFORE
              MOV    A,#46H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.6        ;  SHOW CONTROLLER NOW ON

CTRL6B        JB     B.7,ON6BTST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.7,CTRL6C ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#47H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.7        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6C
ON6BTST       JB     CTRL0.7,CTRL6C ;NOW ON, DONE IF ON BEFORE
              MOV    A,#47H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.7        ;  SHOW CONTROLLER NOW ON

CTRL6C        MOV    P1,#0EFH     ;BRING Sw8 - SwF LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              CPL    A            ;ACTIVE HIGH RESULT
              MOV    B,A          ;SAVE IT IN B

              JB     B.0,ON6CTST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.0,CTRL6D ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#48H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.0        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6D
ON6CTST       JB     CTRL1.0,CTRL6D ;NOW ON, DONE IF ON BEFORE
              MOV    A,#48H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.0        ;  SHOW CONTROLLER NOW ON

CTRL6D        JB     B.1,ON6DTST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.1,CTRL6E ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#49H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.1        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6E
ON6DTST       JB     CTRL1.1,CTRL6E ;NOW ON, DONE IF ON BEFORE
              MOV    A,#49H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.1        ;  SHOW CONTROLLER NOW ON

CTRL6E        JB     B.2,ON6ETST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.2,CTRL6F ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4AH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.2        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL6F
ON6ETST       JB     CTRL1.2,CTRL6F ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4AH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.2        ;  SHOW CONTROLLER NOW ON

CTRL6F        JB     B.3,ON6FTST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.3,CTRL70 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4BH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.3        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL70
ON6FTST       JB     CTRL1.3,CTRL70 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4BH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.3        ;  SHOW CONTROLLER NOW ON

CTRL70        JB     B.4,ON70TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.4,CTRL71 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4CH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.4        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL71
ON70TST       JB     CTRL1.4,CTRL71 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4CH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.4        ;  SHOW CONTROLLER NOW ON

CTRL71        JB     B.5,ON71TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.5,CTRL72 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4DH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.5        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL72
ON71TST       JB     CTRL1.5,CTRL72 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4DH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.5        ;  SHOW CONTROLLER NOW ON

CTRL72        JB     B.6,ON72TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.6,CTRL73 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4EH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.6        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL73
ON72TST       JB     CTRL1.6,CTRL73 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4EH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.6        ;  SHOW CONTROLLER NOW ON

CTRL73        JB     B.7,ON73TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.7,morectrl ;NOW OFF, DONE IF OFF B4
              MOV    A,#4FH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.7        ;  SHOW CONTROLLER NOW OFF
              JMP    morectrl
ON73TST       JB     CTRL1.7,morectrl ;NOW ON, DONE IF ON B4
              MOV    A,#4FH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.7        ;  SHOW CONTROLLER NOW ON

; version 0.2
morectrl      jnb    mode.1,setmode ;if no more controllers
                                    ;selected, skip 'em
              mov    p1,#0fbh       ;bring mode line low
              mov    dptr,#keys
              movx   a,@dptr        ;read transpose values
              cpl    a              ;active high
              mov    b,a            ;SAVE IT IN B

ctrl50        jb     b.0,on50tst    ;NOW ON, SEE IF ON BEFORE
              jnb    ctrl2.0,ctrl51 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#50h         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              clr    ctrl2.0        ;  SHOW CONTROLLER NOW OFF
              jmp    ctrl51
on50tst       jb     ctrl2.0,ctrl51 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#50H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL2.0        ;  SHOW CONTROLLER NOW ON

CTRL51        JB     B.1,ON51TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL2.1,CTRL52 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#51H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL2.1        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL52
ON51TST       JB     CTRL2.1,CTRL52 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#51H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL2.1        ;  SHOW CONTROLLER NOW ON

CTRL52        JB     B.2,ON52TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL2.2,CTRL53 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#52H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL2.2        ;  SHOW CONTROLLER NOW OFF
              JMP    CTRL53
ON52TST       JB     CTRL2.2,CTRL53 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#52H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL2.2        ;  SHOW CONTROLLER NOW ON

CTRL53        JB     B.3,ON53TST    ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL2.3,setmode ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#53H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL2.3        ;  SHOW CONTROLLER NOW OFF
              JMP    setmode
ON53TST       JB     CTRL2.3,setmode ;NOW ON, DONE IF ON BEFORE
              MOV    A,#53H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL2.3        ;  SHOW CONTROLLER NOW ON


;.---------------------------------------------------------.
;|     Allow operator to change between uni-retrigger and  |
;|  uni-legato velocity algorithms.                        |
;`---------------------------------------------------------'

SETMODE       MOV    P1,#0FBH     ;BRING MODE LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              JB     ACC.7,transmode ;NO MODE CHANGE
              CPL    MODE.0       ;CHANGE THE OPERATING MODE
              CPL    P3.0         ;SHOW THE MODE ON PIN 10
SETWAIT       MOVX   A,@DPTR      ;READ THE LATCH AGAIN
              JNB    ACC.7,SETWAIT ;WAIT FOR SWITCH RELEASE

; version 0.2
transmode     movx   a,@dptr       ;read the latch again
              jb     acc.6,wait1ms ;no transmode change
              cpl    mode.1
transwait     movx   a,@dptr
              jnb    acc.6,transwait
;

WAIT1MS       CPL    T1           ;show spin wait on pin 15
              JNB    F0,WAIT1MS   ;WAIT FOR 1 mS FLAG TO
                                  ;BECOME TRUE
              JMP    BREAKTEST


************************************************************
************************************************************
***                                                      ***
**                     SUBROUTINES                        **
***                                                      ***
************************************************************
************************************************************

************************************************************
* CONTROLOFF - TURNS THE CONTROLLER SPECIFIED IN ACC A OFF *
************************************************************
;REQUIRES: ACC A CONTAINS THE CONTROLLER # TO TURN OFF
;RETURNED: NOTHING
;REGS AFF: ACC A CLEARED

CONTROLOFF    PUSH   ACC          ;SAVE ACCUMULATOR
              MOV    A,#0B0H      ;CONTROL CHANGE
              CALL   ENQUEUE
              POP    ACC          ;CONTROLLER #
              CALL   ENQUEUE
              CLR    A            ;CONTROLLER OFF
              CALL   ENQUEUE
              RET


************************************************************
*  CONTROLON - TURNS THE CONTROLLER SPECIFIED IN ACC A ON  *
************************************************************
;REQUIRES: ACC A CONTAINS THE CONTROLLER # TO TURN ON
;RETURNED: NOTHING
;REGS AFF: ACC A TRASHED

CONTROLON     PUSH   ACC          ;SAVE ACCUMULATOR
              MOV    A,#0B0H      ;CONTROL CHANGE
              CALL   ENQUEUE
              POP    ACC          ;CONTROLLER #
              CALL   ENQUEUE
              MOV    A,#7FH       ;CONTROLLER ON
              CALL   ENQUEUE
              RET


************************************************************
*  ENQUEUE - ATTEMPTS TO INSERT BYTE INTO FIFO SERIAL TX   *
*            QUEUE.  IF QUEUE IS FULL, WAITS UNTIL SPACE   *
*            IN QUEUE IS AVAILABLE                         *
************************************************************
;REQUIRES: ACC CONTAINS BYTE TO ENQUEUE
;RETURNS:  NOTHING
;REGS AFFECTED: NONE

ENQUEUE       PUSH   00           ;SAVE R0
              MOV    R0,A         ;PRESERVE ACC
              MOV    A,QHEAD      ;LAST HEAD POINTER
              INC    A            ;NEXT LOCATION
              CJNE   A,#QUETOP+1,ENQUE1 ;NOT FIFO WRAP
              MOV    A,#QUEBOT    ;REPAIR QUEUE WRAP AROUND
ENQUE1        CJNE   A,QTAIL,ENQUE2 ;FALL THRU IF QUEUE FULL

;-------------temporary stuff for testing only--------------
;              push   dpl
;              push   dph
;              MOV    DPTR,#QUEFULL
;              CALL   TXSTRING     ;QUEUE FULL MESSAGE
;              pop    dph
;              pop    dpl
;-----------------------------------------------------------

              SJMP   ENQUE1       ;WAIT FOR SPACE IN QUEUE
ENQUE2        XCH    A,R0         ;SET POINTER & RESTORE ACC
              MOV    @R0,A        ;ENTER BYTE INTO QUEUE
              MOV    QHEAD,R0     ;SAVE HEAD POINTER
              SETB   ES           ;ENABLE SERIAL INTERRUPTS
              POP    00           ;RESTORE R0
              RET                 ;SUCCESS EXIT


************************************************************
* NOTEON - TAKES NOTE VALUE IN ACC A AND VELOCITY IN ACC B *
*          AND ISSUES A NOTE-OFF OF LAST SOUNDING NOTE FOL-*
*          LOWING NOTE-ON. IF A=0, NO NEW NOTE-ON IS SENT, *
*          ONLY A NOTE-OFF. IF LASTNOTE=0, NO NOTE-OFF IS  *
*          SENT, ONLY A NOTE-ON.                           *
************************************************************
;REQUIRES: A CONTAINS NOTE NUMBER (OR 0 FOR NO NOTE-ON)
;          B CONTAINS REQUESTED VELOCITY
;RETURNS:  LASTNOTE RETURNS UPDATED NOTE-ON (OR 0)
;          LASTVEL RETURNS UPDATED VELOCITY
;REGS AFFECTED: none

NOTEON        PUSH   ACC          ;SAVE NOTE VAL (IF ANY)
              PUSH   0            ;SAVE R0
; version 0.2
              mov    r0,a         ;note # in r0
              jz     setforoff    ;no note on
turnon        mov    a,#90h       ;note on
              call   enqueue
              mov    a,r0         ;note #
              add    a,#2fh       ;  base is 30h
              add    a,lastoct    ;  add in octave value
              call   enqueue
              mov    a,b          ;velocity

; VERSION 0.3
              MOV    LASTVEL,A    ;SAVE LAST VELOCITY
;
              call   enqueue
              mov    a,lastnote   ;acc = last note on
              mov    lastnote,r0  ;lastnote = current note on
              jmp    turnoff

setforoff     mov    a,lastnote
turnoff       jz     no_off       ;no note to turn off
              push   acc
              mov    a,#90h       ;note on
              call   enqueue
              pop    acc
              add    a,#2fh
              add    a,lastoct
              call   enqueue      ;note #
              clr    a
              call   enqueue      ;0 velocity
              mov    lastnote,r0  ;save new lastnote
no_off        CLR    A
              MOV    WINDOW,A     ;RESET SENSITIVITY COUNTER
              pop    00
              pop    acc
              RET


************************************************************
* RXBYTE: RETURNS A BYTE FROM DRYICE SERIAL PORT           *
************************************************************
;  INPUT: NONE
;  OUTPUT: RECEIVED BYTE IN A
;  TRASHED: NONE

RXBYTE        PUSH   DPL          ;SAVE DPTR ON STACK
              PUSH   DPH
              MOV    DPTR,#ACIASR ;POINT TO STATUS REG
RXWAIT        MOVX   A,@DPTR      ;READ STATUS BYTE
              ANL    A,#01        ;RDRF FLAG SET?
              JZ     RXWAIT       ;  NO BYTE YET
              MOV    DPTR,#ACIADR ;  YES, SET ADDR OF DR
              MOVX   A,@DPTR      ;RX'D BYTE NOW IN A
              POP    DPH
              POP    DPL          ;RESTORE DPTR
              RET


************************************************************
* TXBYTE: SENDS BYTE FROM ACC VIA DRYICE SERIAL PORT       *
************************************************************
;  INPUT: BYTE TO SEND IN ACC
;  OUTPUT: NONE
;  TRASHED: NONE

TXBYTE        PUSH   DPL          ;SAVE DPTR ON STACK
              PUSH   DPH
              PUSH   ACC          ;SAVE ACC FOR LATER
              MOV    DPTR,#ACIASR ;POINT TO STATUS REGISTER
TXLOOP        MOVX   A,@DPTR      ;READ STATUS REG
              ANL    A,#04H       ;TDRE FLAG SET?
              JZ     TXLOOP       ;  NO, WAIT FOR UART
              POP    ACC          ;  YES, GET ORIGINAL BYTE
              MOV    DPTR,#ACIADR ;POINT TO DATA REG
              MOVX   @DPTR,A
              POP    DPH          ;RESTORE DATA POINTER
              POP    DPL
              RET


************************************************************
* TXSTRING: SENDS NULL TERMINATED STRING VIA SERIAL PORT   *
************************************************************
; REQUIRES: DPTR MUST POINT TO START OF STRING
; RETURNS:  NOTHING
; REGS AFFECTED: NONE

TXSTRING      PUSH   ACC
              PUSH   DPH
              PUSH   DPL
TXSTRING0     CLR    A
              MOVC   A,@A+DPTR    ;READ BYTE
              JZ     TXDONE       ;EXIT IF TRAILING NULL
              CALL   TXBYTE       ;OTHERWISE SEND IT
              INC    DPTR
              JMP    TXSTRING0    ;  AND GET THE NEXT ONE
TXDONE        POP    DPL
              POP    DPH
              POP    ACC
              RET

************************************************************
************************************************************
***                                                      ***
**               POLYPHONIC VERSION                       **
***                                                      ***
************************************************************
************************************************************

POLYINIT      MOV    PSW,#00H     ;USE LOWEST SET OF REGISTERS
              MOV    SP,#STACK    ;SET SP ABOVE INT VARS

;-------------temporary stuff for testing only--------------
;              MOV    A,#0FFH      ;INITIALIZE ALL INTERNAL RAM
;              MOV    R0,#07FH     ;TO FF FOR TESTING PURPOSES
;pRAM_FILL     MOV    @R0,A
;              DJNZ   R0,pRAM_FILL
;-----------------------------------------------------------

              CLR    A
              MOV    KEY0,A       ;INITIALIZE ALL NOTES AS
              MOV    KEY1,A       ;BEING SILENT
              MOV    KEY2,A
              MOV    KEY3,A
              MOV    KEY4,A
              MOV    LASTOCT,A    ;NO INITIAL TRANSPOSE
              MOV    TICK,A
              MOV    CTRL0,A      ;NO CONTROLLERS
              MOV    CTRL1,A

              MOV    A,#7FH       ;SET ALL INITIAL NOTE
              MOV    VEL0,A       ;  STATUS BYTES TO MAX
              MOV    VEL1,A       ;  VELOCITY = 7FH
              MOV    VEL2,A
              MOV    VEL3,A
              MOV    VEL4,A

              MOV    TMOD,#21h    ;DISABLE GATE, TIMER MODE
                                  ;TIMER 1: MODE 2 (8 BIT)
                                  ;TIMER 0: MODE 1 (16 BIT)
              MOV    TL1,#0FFh    ;TMR 1 RELOAD VALUE=255
              MOV    TH1,#0FFh    ;  SINCE SMOD=0 (MIDI BAUD)
              MOV    TL0,#LOW(-1000)  ;TMR 0 RELOAD VALUE=-1000
              MOV    TH0,#HIGH(-1000) ;  FOR 1 kHz 'TICK'
              MOV    TCON,#50h    ;BOTH TIMERS RUNNING
              MOV    SCON,#70h    ;MODE 1, 8 BIT MIDI UART

              MOV    QHEAD,#QUETOP ;INITIALIZE QUEUE POINTERS
              MOV    QTAIL,#QUETOP ;  FOR EMPTY QUEUE
              SETB   EA           ;ENABLE INTERRUPTS
              SETB   ES           ;ENABLE SERIAL INTERRUPTS
              SETB   TI           ;CALL TX ROUTINE TO ARM QUEUE
                                  ;(DOESN'T TRANSMIT ANYTHING
                                  ; BECAUSE QUEUE IS EMPTY)
              SETB   ET0          ;ENABLE TIMER 0 INTERRUPTS
                                  ;(TICK TIMER)


************************************************************
*                    KEYBOARD SCAN                         *
************************************************************
;.---------------------------------------------------------.
;|    Check to see which keys are in the UP position. If a |
;| key is up check to see if it was previously DOWN (VELx=0|
;| for a key down).  If it was previously down, issue a    |
;| NOTE-OFF. If a key is up and NOT previously down, reset |
;| the dynamic velocity (VELx) to max (7F).                |
;`---------------------------------------------------------'

pBREAKTEST    CLR    F0           ;RESET PSW.5 (1 mS FLAG
                                  ;INDICATOR)

              MOV    P1,#0FEH     ;BRING 'BREAK' LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE LATCH VALUE IN B

pBRKTEST0     JB     B.0,pBRKTEST1 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL0        ;READ NOTE VALUE
              JNZ    pVEL7F_0      ;IF VEL <> 0 RESET TO 7F
              MOV    A,#90H        ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,KEY0       ;GET THE PREVIOUS NOTE-ON
                                  ;(BASE NOTE 31H, Index
                                  ; FINGER = 2nd note)
              LCALL  ENQUEUE
              CLR    A            ;VELOCITY = 0 (OFF)
              LCALL  ENQUEUE
pVEL7F_0      MOV    VEL0,#7FH    ;RESET NOTE 0 == 7F

pBRKTEST1     JB     B.1,pBRKTEST2 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL1        ;READ NOTE VALUE
              JNZ    pVEL7F_1      ;IF VEL <> 0 RESET TO 7F
              MOV    A,#90H        ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,KEY1
              LCALL  ENQUEUE
              CLR    A            ;VELOCITY = 0 (OFF)
              LCALL  ENQUEUE
pVEL7F_1      MOV    VEL1,#7FH    ;RESET NOTE 1 == 7F

pBRKTEST2     JB     B.2,pBRKTEST3 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL2        ;READ NOTE VALUE
              JNZ    pVEL7F_2      ;IF VEL <> 0 RESET TO 7F
              MOV    A,#90H        ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,KEY2
              LCALL  ENQUEUE
              CLR    A            ;VELOCITY = 0 (OFF)
              LCALL  ENQUEUE
pVEL7F_2      MOV    VEL2,#7FH   ;RESET NOTE 2 == 7F

pBRKTEST3     JB     B.3,pBRKTEST4 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL3        ;READ NOTE VALUE
              JNZ    pVEL7F_3      ;IF VEL <> 0 RESET TO 7F
              MOV    A,#90H        ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,KEY3
              LCALL  ENQUEUE
              CLR    A            ;VELOCITY = 0 (OFF)
              LCALL  ENQUEUE
pVEL7F_3      MOV    VEL3,#7FH    ;RESET NOTE 3 == 7F

pBRKTEST4     JB     B.4,pMAKETEST ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL4        ;READ NOTE VALUE
              JNZ    pVEL7F_4      ;IF VEL <> 0 RESET TO 7F
              MOV    A,#90H        ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,KEY4
              LCALL  ENQUEUE
              CLR    A            ;VELOCITY = 0 (OFF)
              LCALL  ENQUEUE
pVEL7F_4      MOV    VEL4,#7FH    ;RESET NOTE 4 == 7F

;.---------------------------------------------------------.
;|                                                         |
;`---------------------------------------------------------'

pMAKETEST     MOV    P1,#0FDH     ;BRING 'MAKE' LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE LATCH VALUE IN B

pMKTEST0      JB     B.0,pMKTEST1 ;KEY NOT DOWN, CHECK NEXT
              MOV    A,VEL0       ;READ NOTE VALUE
              JZ     pMKTEST1     ;IF VEL = 0 ALREADY ON
pNOTEON0      MOV    R0,A         ;SAVE ACC
              MOV    A,#90H       ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,#31H       ;BASE NOTE = 31H
              ADD    A,LASTOCT    ;OCTAVE OFFSET
              MOV    KEY0,A       ;SAVE FOR LATER NOTE-OFF
              LCALL  ENQUEUE
              MOV    A,R0         ;VELOCITY
              LCALL  ENQUEUE
              CLR    A
              MOV    VEL0,A       ;RESET VELOCITY = 00

pMKTEST1      JB     B.1,pMKTEST2 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL1       ;READ NOTE VALUE
              JZ     pMKTEST2     ;IF VEL = 0 ALREADY ON
pNOTEON1      MOV    R0,A         ;SAVE ACC
              MOV    A,#90H       ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,#32H       ;NOTE 32H
              ADD    A,LASTOCT    ;OCTAVE OFFSET
              MOV    KEY1,A       ;SAVE FOR LATER NOTE-OFF
              LCALL  ENQUEUE
              MOV    A,R0         ;VELOCITY
              LCALL  ENQUEUE
              CLR    A
              MOV    VEL1,A       ;RESET VELOCITY = 00

pMKTEST2      JB     B.2,pMKTEST3 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL2       ;READ NOTE VALUE
              JZ     pMKTEST3     ;IF VEL = 0 ALREADY ON
pNOTEON2      MOV    R0,A         ;SAVE ACC
              MOV    A,#90H       ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,#33H       ;NOTE 33H
              ADD    A,LASTOCT    ;OCTAVE OFFSET
              MOV    KEY2,A       ;SAVE FOR LATER NOTE-OFF
              LCALL  ENQUEUE
              MOV    A,R0         ;VELOCITY
              LCALL  ENQUEUE
              CLR    A
              MOV    VEL2,A       ;RESET VELOCITY = 00

pMKTEST3      JB     B.3,pMKTEST4 ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL3       ;READ NOTE VALUE
              JZ     pMKTEST4     ;IF VEL = 0 ALREADY ON
pNOTEON3      MOV    R0,A         ;SAVE ACC
              MOV    A,#90H       ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,#34H       ;NOTE 34H
              ADD    A,LASTOCT    ;OCTAVE OFFSET
              MOV    KEY3,A       ;SAVE FOR LATER NOTE-OFF
              LCALL  ENQUEUE
              MOV    A,R0         ;VELOCITY
              LCALL  ENQUEUE
              CLR    A
              MOV    VEL3,A       ;RESET VELOCITY = 00

pMKTEST4      JB     B.4,pCHECKSW ;KEY NOT UP, CHECK NEXT
              MOV    A,VEL4       ;READ NOTE VALUE
              JZ     pCHECKSW     ;IF VEL = 0 ALREADY ON
pNOTEON4      MOV    R0,A         ;SAVE ACC
              MOV    A,#90H       ;ELSE NOTE ON, CH 1
              LCALL  ENQUEUE
              MOV    A,#30H       ;BASE NOTE 30H (THUMB)
              ADD    A,LASTOCT    ;OCTAVE OFFSET
              MOV    KEY4,A       ;SAVE FOR LATER NOTE-OFF
              LCALL  ENQUEUE
              MOV    A,R0         ;VELOCITY
              LCALL  ENQUEUE
              CLR    A
              MOV    VEL4,A       ;RESET VELOCITY = 00


************************************************************
*                 TRANSPOSE SWITCH SCAN                    *
************************************************************
;.---------------------------------------------------------.
;|    Check the TRANSPOSE switches.                        |
;`---------------------------------------------------------'

pCHECKSW      MOV    A,TICK       ;READ SWITCH DEBOUNCE COUNT
              INC    A            ;  TICK += 1
              CJNE   A,#10,pNOTYET ;IF TICK <> 10, MUST WAIT
              CLR    A            ;  IF = 10, CHECK SWITCHES
              MOV    TICK,A       ;  AND RESET TICK = 0
              SJMP   pPLUS24
pNOTYET       MOV    TICK,A       ;DON'T BOTHER WITH SWITCHES
              JMP    pWAIT1MS     ; ... BACK TO KEYBOARD


pPLUS24       MOV    P1,#0FBH     ;BRING TRANSPOSE LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              MOV    B,A          ;SAVE IT IN B

              CLR    A            ;SET NEW OCTAVE DEFAULT TO 0
              JB     B.3,pPLUS12  ;SW NOT CLOSED, SKIP TO NEXT
              MOV    A,#24        ;UP 2 OCTAVES
              JMP    pCOMPARE     ;GO TO COMPARE WITH LASTOCT

pPLUS12       JB     B.2,pMINUS12
              MOV    A,#12        ;UP 1 OCTAVE
              JMP    pCOMPARE

pMINUS12      JB     B.1,pMINUS24
              MOV    A,#-12       ;DOWN 1 OCTAVE
              JMP    pCOMPARE

pMINUS24      JB     B.0,pCOMPARE
              MOV    A,#-24       ;DOWN 2 OCTAVES

pCOMPARE      MOV    LASTOCT,A    ;SAVE TRANSPOSE VALUE

************************************************************
*                CONTROLLER SWITCH SCAN                    *
************************************************************

pCONTROL      MOV    P1,#0F7H     ;BRING Sw0 - Sw7 LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              CPL    A            ;ACTIVE HIGH RESULT
              MOV    B,A          ;SAVE IT IN B

pCTRL64       JB     B.0,pON64TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.0,pCTRL65 ;NOW OFF, DONE IF OFF B4
              MOV    A,#40H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.0        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL65
pON64TST      JB     CTRL0.0,pCTRL65 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#40H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.0        ;  SHOW CONTROLLER NOW ON

pCTRL65       JB     B.1,pON65TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.1,pCTRL66 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#41H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.1        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL66
pON65TST      JB     CTRL0.1,pCTRL66 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#41H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.1        ;  SHOW CONTROLLER NOW ON

pCTRL66       JB     B.2,pON66TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.2,pCTRL67 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#42H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.2        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL67
pON66TST      JB     CTRL0.2,pCTRL67 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#42H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.2        ;  SHOW CONTROLLER NOW ON

pCTRL67       JB     B.3,pON67TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.3,pCTRL68 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#43H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.3        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL68
pON67TST      JB     CTRL0.3,pCTRL68 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#43H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.3        ;  SHOW CONTROLLER NOW ON

pCTRL68       JB     B.4,pON68TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.4,pCTRL69 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#44H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.4        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL69
pON68TST      JB     CTRL0.4,pCTRL69 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#44H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.4        ;  SHOW CONTROLLER NOW ON

pCTRL69       JB     B.5,pON69TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.5,pCTRL6A ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#45H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.5        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6A
pON69TST      JB     CTRL0.5,pCTRL6A ;NOW ON, DONE IF ON BEFORE
              MOV    A,#45H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.5        ;  SHOW CONTROLLER NOW ON

pCTRL6A       JB     B.6,pON6ATST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.6,pCTRL6B ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#46H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.6        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6B
pON6ATST      JB     CTRL0.6,pCTRL6B ;NOW ON, DONE IF ON BEFORE
              MOV    A,#46H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.6        ;  SHOW CONTROLLER NOW ON

pCTRL6B       JB     B.7,pON6BTST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL0.7,pCTRL6C ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#47H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL0.7        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6C
pON6BTST      JB     CTRL0.7,pCTRL6C ;NOW ON, DONE IF ON BEFORE
              MOV    A,#47H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL0.7        ;  SHOW CONTROLLER NOW ON


pCTRL6C       MOV    P1,#0EFH     ;BRING Sw8 - SwF LINE LOW
              MOV    DPTR,#KEYS   ;POINT TO KYBD LATCH
              MOVX   A,@DPTR      ;READ THE LATCH
              CPL    A            ;ACTIVE HIGH RESULT
              MOV    B,A          ;SAVE IT IN B

              JB     B.0,pON6CTST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.0,pCTRL6D ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#48H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.0        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6D
pON6CTST      JB     CTRL1.0,pCTRL6D ;NOW ON, DONE IF ON BEFORE
              MOV    A,#48H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.0        ;  SHOW CONTROLLER NOW ON

pCTRL6D       JB     B.1,pON6DTST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.1,pCTRL6E ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#49H         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.1        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6E
pON6DTST      JB     CTRL1.1,pCTRL6E ;NOW ON, DONE IF ON BEFORE
              MOV    A,#49H         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.1        ;  SHOW CONTROLLER NOW ON

pCTRL6E       JB     B.2,pON6ETST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.2,pCTRL6F ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4AH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.2        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL6F
pON6ETST      JB     CTRL1.2,pCTRL6F ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4AH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.2        ;  SHOW CONTROLLER NOW ON

pCTRL6F       JB     B.3,pON6FTST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.3,pCTRL70 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4BH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.3        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL70
pON6FTST      JB     CTRL1.3,pCTRL70 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4BH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.3        ;  SHOW CONTROLLER NOW ON

pCTRL70       JB     B.4,pON70TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.4,pCTRL71 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4CH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.4        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL71
pON70TST      JB     CTRL1.4,pCTRL71 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4CH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.4        ;  SHOW CONTROLLER NOW ON

pCTRL71       JB     B.5,pON71TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.5,pCTRL72 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4DH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.5        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL72
pON71TST      JB     CTRL1.5,pCTRL72 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4DH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.5        ;  SHOW CONTROLLER NOW ON

pCTRL72       JB     B.6,pON72TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.6,pCTRL73 ;NOW OFF, DONE IF OFF BEFORE
              MOV    A,#4EH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.6        ;  SHOW CONTROLLER NOW OFF
              JMP    pCTRL73
pON72TST      JB     CTRL1.6,pCTRL73 ;NOW ON, DONE IF ON BEFORE
              MOV    A,#4EH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.6        ;  SHOW CONTROLLER NOW ON

pCTRL73       JB     B.7,pON73TST   ;NOW ON, SEE IF ON BEFORE
              JNB    CTRL1.7,pCHG_SENS ;NOW OFF, DONE IF OFF B4
              MOV    A,#4FH         ;  NOW OFF, WAS ON -> DO OFF
              CALL   CONTROLOFF
              CLR    CTRL1.7        ;  SHOW CONTROLLER NOW OFF
              JMP    pCHG_SENS
pON73TST      JB     CTRL1.7,pCHG_SENS ;NOW ON, DONE IF ON B4
              MOV    A,#4FH         ;  NOW ON, WAS OFF -> DO ON
              CALL   CONTROLON
              SETB   CTRL1.7        ;  SHOW CONTROLLER NOW ON


pCHG_SENS     NOP                  ;NO SENSITIVITY ADJUSTMENT
                                   ;FOR POLYPHONIC MODE

pWAIT1MS      cpl    T1           ;show spin wait on pin 15
              JNB    F0,pWAIT1MS  ;WAIT FOR 1 mS FLAG TO
                                  ;BECOME TRUE

              JMP   pBREAKTEST




************************************************************
************************************************************
************************************************************

QUEFULL       DB     '\r\nMIDI Queue FULL!\r\n',0


          END

